// Billboards.cpp: implementation of the CBillboards class.
//
//////////////////////////////////////////////////////////////////////

#include "Billboards.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBillboards::CBillboards()
{
	billboardMatrix = GetIdentityMatrix();
	i = 0;

	time_last = 0;
}

CBillboards::~CBillboards()
{
	for(int i = 0; i < Textures.size(); i++)
	{
		SAFE_RELEASE(Textures.at(i));
	}

}

bool CBillboards::InitializeBillboard(LPDIRECT3DDEVICE8 device, std::vector<char*> TextureFilenames)
{

	time_last	 = timeGetTime();

	LPDIRECT3DTEXTURE8 tempTexture;

	for(int i = 0; i < TextureFilenames.size(); i ++)
	{
		HRESULT hr = D3DXCreateTextureFromFileEx(device, TextureFilenames.at(i), D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT,
							0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,D3DX_DEFAULT, D3DX_DEFAULT,D3DCOLOR_XRGB(0,0,0), NULL, 
							NULL, &tempTexture);
		if(hr != D3D_OK)
		{
			LogError("texture at %s did not load", TextureFilenames.at(i));

			tempTexture = NULL;
		}

		Textures.push_back(tempTexture);
	}
	
	return true;
}

bool CBillboards::DrawBillboard(LPDIRECT3DDEVICE8 device, D3DXMATRIX ViewMatrix, D3DXVECTOR3 Rotation, Billboard *billboard)
{
	long			time_current = timeGetTime();
	long			elapsed_ms   = time_current - time_last;

	LPDIRECT3DTEXTURE8 tempTexture;
	D3DXMATRIX rotate;
	D3DXMATRIX temp = ViewMatrix;


	D3DXMatrixInverse(&temp, NULL, &ViewMatrix);

	temp._41 = billboard->Location.x;
	temp._42 = billboard->Location.y;
	temp._43 = billboard->Location.z;

	if(billboard->RotateY == false)
	{
		D3DXMatrixRotationX(&rotate, Rotation.x );

		D3DXMatrixMultiply(&temp, &rotate, &temp);
	}

	int blah = Textures.size();

	if(elapsed_ms > billboard->Delay) 
	{
		if(i < Textures.size())
		{
			i++;
		}

		if(i == Textures.size())
		{
			i = 0;
		}

		time_last = time_current;
	}

	tempTexture = Textures.at(i);

	//set world matrix 
	device->SetTransform(D3DTS_WORLD, &temp);

	//turn off lighting
	device->SetRenderState(D3DRS_LIGHTING, FALSE);

	//turn off depth buffer writes
	device->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);

	//set the vertex streams
	device->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1);

	//disable detail mapping
	device->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);


	LitVertex Mesh[4];
	DWORD Color = billboard->Color;

	Mesh[0].x = -billboard->Size.x / 2; Mesh[0].y = billboard->Size.y;  Mesh[0].z = 0;
	Mesh[0].u = 0.0f;  Mesh[0].v = 0.0f;
	Mesh[0].color = Color;

	Mesh[1].x = billboard->Size.x / 2;  Mesh[1].y = billboard->Size.y;	Mesh[1].z = 0;
	Mesh[1].u = 1.0f;  Mesh[1].v = 0.0f;
	Mesh[1].color = Color;

	Mesh[2].x = -billboard->Size.x / 2;  Mesh[2].y = 0; Mesh[2].z = 0;
	Mesh[2].u = 0.0f;   Mesh[2].v = 1.0f;
	Mesh[2].color = Color;

	Mesh[3].x = billboard->Size.x / 2; Mesh[3].y = 0; Mesh[3].z = 0;
	Mesh[3].u = 1.0f; Mesh[3].v = 1.0f;
	Mesh[3].color = Color;
	
	device->SetTexture(0, tempTexture);
	device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));


	//restore settings
	device->SetRenderState(D3DRS_LIGHTING, TRUE);
	device->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
	device->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_MODULATE2X);

	return true;
}

