// Camera.cpp: implementation of the CCamera class.
//
//////////////////////////////////////////////////////////////////////
#pragma warning( disable : 4244 )

#include "Camera.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCamera::CCamera()
{
	ZeroMemory(&projection, sizeof(projection));
	ZeroMemory(&view, sizeof(view));

	ZeroMemory(&Position, sizeof(Position));
	ZeroMemory(&Rotation, sizeof(Rotation));

	ZeroMemory(&Eye, sizeof(Eye));

	At.x = 0;
	At.y = 0;
	At.z = 1;

	Up.x = 0;
	Up.y = 1;
	Up.z = 0;

	fieldViewX	= 75;
	fieldViewY	= 75;
	nearPlane	= .25f;
	farPlane	= 10000;

	moveVertical = true;
}

CCamera::~CCamera()
{

}

bool CCamera::InitializeCamera(HWND hwnd, LPDIRECT3DDEVICE8 device)
{

	SetProjectionMatrix(hwnd, device);

	SetViewMatrix(&At,&Up,&Eye, device);


	return true;
}

void CCamera::SetProjectionMatrix(HWND hwnd, LPDIRECT3DDEVICE8 device)
{
	RECT bounds;
    GetClientRect(hwnd, &bounds);

	float AspectRatio = (float)((bounds.right - bounds.left) / (bounds.bottom - bounds.top));

	float w = (float)(1/tan(EIMATH_DEGTORAD(fieldViewX) * 0.5)) / AspectRatio;  
	float h = (float)(1/tan(EIMATH_DEGTORAD(fieldViewY) * 0.5)) / AspectRatio;
	float q = (float) farPlane/(farPlane - nearPlane);


	projection(0,0) = w;
	projection(1,1) = h;
	projection(2,2) = q;
	projection(2,3) = 1;
	projection(3,2) = - q *nearPlane;

	device->SetTransform(D3DTS_PROJECTION, &projection);

}

void CCamera::SetViewMatrix(D3DXVECTOR3 *pAt,D3DXVECTOR3 *pUp,D3DXVECTOR3 *pEye, LPDIRECT3DDEVICE8 device)
{

	D3DXVECTOR3	XAxis;
	D3DXVECTOR3	YAxis;
	D3DXVECTOR3	ZAxis;

    // Get the z basis vector, which points straight ahead; the
    // difference from the eye point to the look-at point. This is the 
    // direction of the gaze (+z).
    D3DXVec3Subtract(&ZAxis, pAt, pEye);
    D3DXVec3Normalize(&ZAxis, &ZAxis);

    // Compute the orthogonal axes from the cross product of the gaze 
    // and the pUp vector.
    D3DXVec3Cross(&XAxis, pUp, &ZAxis);
    D3DXVec3Normalize(&XAxis, &XAxis);

    D3DXVec3Cross(&YAxis, &ZAxis, &XAxis);

    // Start building the matrix. The first three rows contain the 
    // basis vectors used to rotate the view to point at the look-at 
    // point. The fourth row contains the translation values. 
    // Rotations are still about the eyepoint.
    view._11 = XAxis.x;
    view._21 = XAxis.y;
    view._31 = XAxis.z;
    view._41 = -D3DXVec3Dot(&XAxis, pEye);

    view._12 = YAxis.x;
    view._22 = YAxis.y;
    view._32 = YAxis.z;
    view._42 = -D3DXVec3Dot(&YAxis, pEye);

    view._13 = ZAxis.x;
    view._23 = ZAxis.y;
    view._33 = ZAxis.z;
    view._43 = -D3DXVec3Dot(&ZAxis, pEye);

	view._14 = 0.0f;    
    view._24 = 0.0f;   
    view._34 = 0.0f;   
    view._44 = 1.0f;

	device->SetTransform(D3DTS_VIEW, &view);

}


void CCamera::RotateXAxis(float rads)
{
	Rotation.x -= rads;
}

void CCamera::RotateYAxis(float rads)
{
	Rotation.y += rads;
}

void CCamera::MoveForward(float velocity)
{
	Position.x += cos(Rotation.x) * sin(Rotation.y) * velocity;

	Position.z += cos(Rotation.x) * cos(Rotation.y) * velocity;

	if(moveVertical == true) 
		Position.y += sin(Rotation.x) * velocity;

}

void CCamera::MoveBackward(float velocity)
{
	Position.x -= cos(Rotation.x) * sin(Rotation.y) * velocity;

	Position.z -= cos(Rotation.x) * cos(Rotation.y) * velocity;

	if(moveVertical == true) 
		Position.y -= sin(Rotation.x) * velocity;

}
	
void CCamera::StrafeLeft(float velocity)
{
	Position.x += cos(Rotation.x) * sin((Rotation.y - EIMATH_PI_2)) * velocity;

	Position.z += cos(Rotation.x) * cos((Rotation.y - EIMATH_PI_2)) * velocity;
}

void CCamera::StrafeRight(float velocity)
{
	Position.x += cos(Rotation.x) * sin((Rotation.y + EIMATH_PI_2)) * velocity;

	Position.z += cos(Rotation.x) * cos((Rotation.y + EIMATH_PI_2)) * velocity;
}

void CCamera::MoveDown(float velocity)
{
	Position.y -= velocity;
}


void CCamera::MoveUp(float velocity)
{
	Position.y += velocity;
}


void CCamera::MoveCameraToLocation(float x, float y, float z)
{
	Position.x = x;
	Position.y = y;
	Position.z = z;	
}

void CCamera::UpdateView(LPDIRECT3DDEVICE8 device)
{
	Eye.x = Position.x;
	Eye.y = Position.y;
	Eye.z = Position.z;
	
	At.x = Position.x + cos(Rotation.x) * sin(Rotation.y);
	At.y = Position.y + sin(Rotation.x);
	At.z = Position.z + cos(Rotation.x) * cos(Rotation.y);

	SetViewMatrix(&At,&Up, &Eye, device);
}


D3DXMATRIX CCamera::GetViewMatrix()
{
	return view;
}


D3DXVECTOR3 CCamera::GetRotationVector()
{
	return Rotation;
}

D3DXVECTOR3 CCamera::GetPositionVector()
{
	return Position;
}
