// Camera.h: interface for the CCamera class.
//
//////////////////////////////////////////////////////////////////////
#if !defined(AFX_CAMERA_H)
#define AFX_CAMERA_H

#include <d3d8.h>
#include <d3dx8.h>

#include "logging.h"
#include "defines.h"

class CCamera 
{
public:
	CCamera();
	virtual ~CCamera();

	bool		InitializeCamera(HWND hwnd, LPDIRECT3DDEVICE8 device);

	void		RotateXAxis(float rads);

	void		RotateYAxis(float rads);

	void		RotateZAxis(float rads);

	void		MoveForward(float velocity);

	void		MoveBackward(float velocity);

	void		MoveUp(float velocity);

	void		MoveDown(float velocity);

	void		StrafeLeft(float velocity);

	void		StrafeRight(float velocity);

	void		MoveCameraToLocation(float x, float y, float z);
	
	void		UpdateView(LPDIRECT3DDEVICE8 device);

	D3DXMATRIX   GetViewMatrix();

	D3DXVECTOR3	 GetRotationVector();

	D3DXVECTOR3	 GetPositionVector();

private:

	void	SetProjectionMatrix(HWND hwnd, LPDIRECT3DDEVICE8 device);

	void	SetViewMatrix(D3DXVECTOR3 *pAt,D3DXVECTOR3 *pUp, D3DXVECTOR3 *pEye, 
							LPDIRECT3DDEVICE8 device);

	LPDIRECT3DDEVICE8 pDevice;

	D3DXMATRIX	projection;
	D3DXMATRIX	view;

	D3DXVECTOR3 At;
	D3DXVECTOR3 Up;
	D3DXVECTOR3 Eye;
	
	D3DXVECTOR3	Position;
	D3DXVECTOR3	Rotation;

	float fieldViewX;
	float fieldViewY;
	float nearPlane;
	float farPlane;

	bool moveVertical;

};

#endif