// Game.cpp: implementation of the CGame class.
//
//////////////////////////////////////////////////////////////////////
//WRITTEN BY: BRIAN PETERSON 
//EMAIL ADDRESS: GOOMYMAN@AOL.COM
//DATE FINISHED: JUNE 2ND 2002
//JOB STATUS: STUDENT ( hire me :D )
//
//COMMENTS: This is the start of my first 3D engine all written by me.
// If you like this engine feel free to use it and if you want drop me 
// an e-mail because I would love to hear if you like it and use it.
// Any questions, comments, looking for a new version or want to report 
// a bug e mail me.
//
//FUTURE VERSIONS: .ZCB file loader, OctTree or BSB technology, Collision Detection,
//	Only drawing what is currently in view using some method, and Animation. 
//  Very Far future may include a GUI.
//
//CODING COMMENTS: There is a lot of error checking to be desired. You will 
// get game crashing errors if the textures or filenames are not in the right 
// spot. Their is a possiblity that you wont be able to move on really fast 
// computers if you get more than 1000 frames per second ( not positive ). 
// Everything can be coded from from the Game.cpp File. If you wish to change
// Screen Size and or full screen resolutions it is found at the very top of Main.cpp.
// Logging is currently disabled. Most of the errors will be reported in 
// the error.txt and info.txt files if turned on in the logging.cpp file.
//
//////////////////////////////////////////////////////////////////////////////////

#include "Game.h"

#include <stdlib.h>
#include <stdio.h>
#include <time.h>



CGame::CGame()
{
	sensitivity = 700;
	keyboardspeed = 125;

	//STUFF BELOW HERE

	LightningTime = 0;
	playing = false;
	loopagain = false;
	RainMore = 0;
	WindSpeed = 1000;
	RainDelay = 0;
	FireVisible = false;

	// LIGHT INFO
	ZeroMemory( &pointLight, sizeof(D3DLIGHT8) );
	pointLight.Type = D3DLIGHT_POINT;
	pointLight.Attenuation1 = 1;
	pointLight.Position.x = 0;
	pointLight.Position.y = 1;
	pointLight.Position.z = 6.5;
	pointLight.Diffuse.r = 100;
	pointLight.Diffuse.g = 100;
	pointLight.Diffuse.b = 100;
	pointLight.Range = 5;

	ZeroMemory( &ambientLight, sizeof(D3DLIGHT8) );
	ambientLight.Type = D3DLIGHT_DIRECTIONAL;
	ambientLight.Ambient.r = .25;
	ambientLight.Ambient.g = .25;
	ambientLight.Ambient.b = .5;
	ambientLight.Direction.x = 0;
	ambientLight.Direction.y = -1;
	ambientLight.Direction.z = 0;

	// X FILE INFO
	tiger.filename = ".\\XFiles\\tiger.x";
	strcpy(tiger.filenameLoc, ".\\XFiles\\");
	tiger.WorldMatrix = GetIdentityMatrix();
	tiger.WorldMatrix._43 = 7;
	tiger.WorldMatrix._41 = 3;
	tiger.WorldMatrix = RotateYawPitchRoll(tiger.WorldMatrix,75,0,0);

	tiger2.filename = ".\\XFiles\\tiger.x";
	strcpy(tiger2.filenameLoc, ".\\XFiles\\");
	tiger2.WorldMatrix = GetIdentityMatrix();
	tiger2.WorldMatrix._41 = -3;
	tiger2.WorldMatrix._43 = 6;
	tiger2.WorldMatrix = RotateYawPitchRoll(tiger2.WorldMatrix,300,0,0);

	//Particle Smoke
	Smoke.TextureFilename = ".\\Textures\\smoke.bmp";
	Smoke.Radius.x = .5;
	Smoke.Radius.y = .5;
	Smoke.Radius.z = .5;
	Smoke.MaxRandParticleDirection.x = 0.5;
	Smoke.MinRandParticleDirection.x = -.5;
	Smoke.MaxRandParticleDirection.y = 1;
	Smoke.MinRandParticleDirection.y = .75;
	Smoke.MaxRandParticleDirection.z = .5;
	Smoke.MinRandParticleDirection.z = -.5;
	Smoke.MaxRandVelocity = 8;
	Smoke.MinRandVelocity = 4;
	Smoke.MaxParticles = 500;
	Smoke.NumParticlesToEmit = 5;
	Smoke.ParticleSize = 1.5;
	Smoke.ScaleX = 1;
	Smoke.ScaleY = 1;
	Smoke.ScaleZ = 25;
	Smoke.MaxLifeTime = 2500;
	Smoke.Position.x = -.25;
	Smoke.Position.y = 0;
	Smoke.Position.z = 6.4;
	Smoke.StartColor = COLOR_RGBA(255,175,175,80);
	Smoke.EndColor = COLOR_RGBA(255,255,255,0);
	Smoke.ColorFade = COLOR_RGBA(0,50,50,10);
	Smoke.AirResitance = 0;
	Smoke.Gravity = 10;
	Smoke.Delay = 100;
	Smoke.Active = false;

	//rain
	Rain.TextureFilename = ".\\Textures\\RainDrop.bmp";
	Rain.Radius.x = 50;
	Rain.Radius.y = 0;
	Rain.Radius.z = 30;
	Rain.MaxRandParticleDirection.x = 0;
	Rain.MinRandParticleDirection.x = 0;
	Rain.MaxRandParticleDirection.y = -1;
	Rain.MinRandParticleDirection.y = -1;
	Rain.MaxRandParticleDirection.z = 0;
	Rain.MinRandParticleDirection.z = 0;
	Rain.MaxRandVelocity = 250;
	Rain.MinRandVelocity = 220;
	Rain.MaxParticles = 10000;
	Rain.NumParticlesToEmit = 4;
	Rain.ParticleSize = 1;
	Rain.ScaleX = 0;
	Rain.ScaleY = 0;
	Rain.ScaleZ = 1;
	Rain.MaxLifeTime = 3000;
	Rain.Position.x = -20;
	Rain.Position.y = 35;
	Rain.Position.z = 0;
	Rain.StartColor = COLOR_RGBA(150,150,150,75);
	Rain.EndColor = COLOR_RGBA(150,150,150,75);
	Rain.ColorFade = COLOR_RGBA(1,1,1,1);
	Rain.AirResitance = 0;
	Rain.Gravity = 0;
	Rain.Delay = 25;
	Rain.Active = true;

	Explotion.TextureFilename = ".\\Textures\\Particle.bmp";
	Explotion.Radius.x = .1;
	Explotion.Radius.y = 0;
	Explotion.Radius.z = .1;
	Explotion.MaxRandParticleDirection.x = 1;
	Explotion.MinRandParticleDirection.x = -1;
	Explotion.MaxRandParticleDirection.y = 1;
	Explotion.MinRandParticleDirection.y = .9;
	Explotion.MaxRandParticleDirection.z = 1;
	Explotion.MinRandParticleDirection.z = -1;
	Explotion.MaxRandVelocity = 12;
	Explotion.MinRandVelocity = 5;
	Explotion.MaxParticles = 250;
	Explotion.NumParticlesToEmit = 50;
	Explotion.ParticleSize = .3;
	Explotion.ScaleX = 5;
	Explotion.ScaleY = 5;
	Explotion.ScaleZ = 5;
	Explotion.MaxLifeTime = 1500;
	Explotion.Position.x = 0;
	Explotion.Position.y = -.7;
	Explotion.Position.z = 6.5;
	Explotion.StartColor = COLOR_RGBA(255,255,255,25);
	Explotion.EndColor = COLOR_RGBA(255,0,0,75);
	Explotion.ColorFade = COLOR_RGBA(1,100,100,50);
	Explotion.AirResitance = .5;
	Explotion.Gravity = 100;
	Explotion.Delay = 10;
	Explotion.Active = false;

	Fire.Location.x = 0;
	Fire.Location.y = -1;
	Fire.Location.z = 6.5;
	Fire.Size.x = 2;
	Fire.Size.y = 3;
	Fire.Delay = 175;
	Fire.Color = COLOR_RGBA(255,255,255,75);
	Fire.RotateY = false;
	Fire.texturenames.push_back(".\\Textures\\flame0.bmp");
	Fire.texturenames.push_back(".\\Textures\\flame1.bmp");
	Fire.texturenames.push_back(".\\Textures\\flame2.bmp");
	Fire.texturenames.push_back(".\\Textures\\flame3.bmp");
	Fire.texturenames.push_back(".\\Textures\\flame4.bmp");
	Fire.texturenames.push_back(".\\Textures\\flame5.bmp");
	Fire.texturenames.push_back(".\\Textures\\flame6.bmp");
	Fire.texturenames.push_back(".\\Textures\\flame7.bmp");

	Lightning.Location.x = 0;
	Lightning.Location.y = -.5;
	Lightning.Location.z = 6.5;
	Lightning.Size.x = 1;
	Lightning.Size.y = 10;
	Lightning.Delay = 100;
	Lightning.Color = COLOR_RGBA(255,255,255,50);
	Lightning.RotateY = false;
	Lightning.texturenames.push_back(".\\Textures\\lightning1.bmp");
	Lightning.texturenames.push_back(".\\Textures\\lightning2.bmp");
	Lightning.texturenames.push_back(".\\Textures\\lightning3.bmp");
	Lightning.texturenames.push_back(".\\Textures\\lightning4.bmp");

	Log.Location.x = 0;
	Log.Location.y = -1;
	Log.Location.z = 6.5;
	Log.Size.x = 1.2;
	Log.Size.y = 1.2;
	Log.Color = COLOR_RGBA(80,80,80,255);
	Log.RotateY = true;
	Log.texturenames.push_back(".\\Textures\\log.bmp");

	Cloud.Location.x = 4;
	Cloud.Location.y = 8;
	Cloud.Location.z = 6.5;
	Cloud.Size.x = 10;
	Cloud.Size.y = 4;
	Cloud.Color = COLOR_RGBA(110,110,125,70);
	Cloud.RotateY = true;
	Cloud.texturenames.push_back(".\\Textures\\cloud.bmp");

	Cloud1.Location.x = 4;
	Cloud1.Location.y = 9;
	Cloud1.Location.z = 15;
	Cloud1.Size.x = 10;
	Cloud1.Size.y = 5;
	Cloud1.Color = COLOR_RGBA(125,125,125,60);
	Cloud1.RotateY = true;
	Cloud1.texturenames.push_back(".\\Textures\\cloud.bmp");

	Cloud2.Location.x = 7;
	Cloud2.Location.y = 10;
	Cloud2.Location.z = 20;
	Cloud2.Size.x = 10;
	Cloud2.Size.y = 5;
	Cloud2.Color = COLOR_RGBA(125,125,125,60);
	Cloud2.RotateY = true;
	Cloud2.texturenames.push_back(".\\Textures\\cloud.bmp");

	Cloud3.Location.x = 15;
	Cloud3.Location.y = 12;
	Cloud3.Location.z = 22;
	Cloud3.Size.x = 14;
	Cloud3.Size.y = 6;
	Cloud3.Color = COLOR_RGBA(110,110,125,60);
	Cloud3.RotateY = true;
	Cloud3.texturenames.push_back(".\\Textures\\cloud.bmp");

}

CGame::~CGame()
{

	SAFE_RELEASE(device);
	SAFE_RELEASE(d3d);

	SAFE_RELEASE(detailmap);

	keyboard->Unacquire();
	mouse->Unacquire();

	SAFE_RELEASE(dInput);
	SAFE_RELEASE(keyboard);
	SAFE_RELEASE(mouse);

	//STUFF BELOW HERE

	PlaySound(NULL, 0, NULL);
	
}

bool CGame::InitializeInput(HINSTANCE hInstance, HWND hwnd)
{

	//For direct input
	if(FAILED(DirectInput8Create(hInstance,DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&dInput,0)))
	{
		return false;
	}

	if(FAILED(dInput->CreateDevice(GUID_SysKeyboard, &keyboard, NULL)))
	{
		return false;
	}

	if(FAILED(dInput->CreateDevice(GUID_SysMouse, &mouse, NULL)))
	{
		return false;
	}

	keyboard->SetDataFormat(&c_dfDIKeyboard);

	keyboard->SetCooperativeLevel(hwnd,DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);

	mouse->SetDataFormat(&c_dfDIMouse2);

	mouse->SetCooperativeLevel(hwnd, DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);

	keyboard->Acquire();
	mouse->Acquire();

	return true;

}

bool CGame::InitializeGame(HINSTANCE hInstance, HWND hwnd, int screen_width, int screen_height, bool fullscreen)
{
	CD3DInitialize		myD3DInitialize;

	if(!myD3DInitialize.InitializeD3D(hwnd,screen_width,screen_height,fullscreen,device,d3d))
	{
		return false;
	}
	if(!InitializeInput(hInstance, hwnd))
	{
		return false;
	}

	myCamera.InitializeCamera(hwnd, device);

	myRenderStates.InitializeRenderStates(device);

	//load detail map 
	D3DXCreateTextureFromFileA(device,".\\Textures\\detail.bmp", &detailmap);

	device->SetTexture(1, detailmap);

	/////////////////////////////////////////////////////////////////
	//Insert Stuff here if you want it to load at start up or run once


		mySkyBox.InitializeSkybox(device,".\\Textures\\desnoonft.bmp",".\\Textures\\desnoonbk.bmp",
			".\\Textures\\desnoonlf.bmp",".\\Textures\\desnoonrt.bmp",".\\Textures\\desnoonup.bmp",
			".\\Textures\\desnoondn.bmp");

		myBillboards.InitializeBillboard(device, Fire.texturenames);
		myBillboards2.InitializeBillboard(device, Log.texturenames);
		myBillboards3.InitializeBillboard(device, Lightning.texturenames);
		myBillboards4.InitializeBillboard(device, Cloud.texturenames);
		myBillboards5.InitializeBillboard(device, Cloud1.texturenames);
		myBillboards6.InitializeBillboard(device, Cloud2.texturenames);
		myBillboards7.InitializeBillboard(device, Cloud3.texturenames);

		myParticleSystem.Initialize(device, &Smoke);
		myParticleSystem2.Initialize(device, &Rain);
		myParticleSystem3.Initialize(device, &Explotion);

		myRenderStates.CreateLight(&ambientLight, device);
		myRenderStates.CreateLight(&pointLight, device);
		myRenderStates.LightOff(pointLight.LightNumber, device);

		myXFile.LoadXFile(&tiger, device);
		myXFile2.LoadXFile(&tiger2, device);

		myCamera.MoveCameraToLocation(.5,2.5,-5);
		myCamera.UpdateView(device);

		PlaySound(".\\Sounds\\rainshower2.wav", 0, SND_FILENAME | SND_LOOP | SND_ASYNC );


	return true;
}


bool CGame::Process()
{

	DWORD  SkyColor = COLOR_RGBA(75,75,75,255);
	
	bool LightningVisible = false;

	static long time_last = timeGetTime();
	long time_current = timeGetTime();
	long elapsed_ms = time_current - time_last;
	if(elapsed_ms == 0)
		elapsed_ms = 1;
	time_last = time_current;

	RainDelay += elapsed_ms;
	LightningTime += elapsed_ms;

	if(LightningTime > 100000)
		LightningTime = 100000;


	CheckKeyboardInput(elapsed_ms);
	CheckMouseInput();


	//EVERYTHING BELOW HERE ( GAME CODE + DRAWING TO SCREEN )
	if(RainDelay > 800 && RainMore < 15)
	{
		RainMore++;

		RainDelay = 0;
		
		Rain.NumParticlesToEmit += 1;
			
		Rain.MinRandParticleDirection.x -= .03;
		Rain.MaxRandParticleDirection.x -= .032;

	}
	else if(RainDelay > 1000 && RainMore > 14)
	{
		RainDelay = 0;

		if(Rain.NumParticlesToEmit > 2)
		{
			Rain.NumParticlesToEmit -= 1;
		}
		if(Rain.MinRandParticleDirection.x < -.2)
		{
			Rain.MinRandParticleDirection.x += .02;
			Rain.MaxRandParticleDirection.x += .021;
		}

	}


	if(LightningTime > 15000)
	{
		LightningVisible = true;
		SkyColor = COLOR_RGBA(175,175,175,255);

		if(playing == false)
		{
			PlaySound(".\\Sounds\\thunderrumble3.wav",0, SND_FILENAME | SND_ASYNC);
			playing = true;
		}

		if(LightningTime > 15400)
		{
			Explotion.Active = true;
			LightningVisible = false;
			FireVisible = true;
			Smoke.Active = true;
			myRenderStates.LightOn(pointLight.LightNumber, device);
		}

		if(LightningTime > 15700)
		{
			SkyColor = COLOR_RGBA(75,75,75,255);
			Explotion.Active = false;
		}

		if(LightningTime > 21000)
		{
			if(loopagain == false)
			{
				PlaySound(".\\Sounds\\rainshower.wav", 0, SND_FILENAME | SND_LOOP | SND_ASYNC);
				loopagain = true;
			}
		}

	}

	Cloud.Location.x -= (float)elapsed_ms / (WindSpeed + 2400);
	Cloud1.Location.x -= (float)elapsed_ms / (WindSpeed + 700);
	Cloud2.Location.x -= (float)elapsed_ms / (WindSpeed + 1000);
	Cloud3.Location.x -= (float)elapsed_ms / WindSpeed;



	myParticleSystem.UpdateParticles(&Smoke, elapsed_ms);
	myParticleSystem.EmitParticles(&Smoke);

	myParticleSystem2.UpdateParticles(&Rain, elapsed_ms);
	myParticleSystem2.EmitParticles(&Rain);

	myParticleSystem3.UpdateParticles(&Explotion, elapsed_ms);
	myParticleSystem3.EmitParticles(&Explotion);


	device->Clear(0,NULL,D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,COLOR_RGBA(0,0,0,0),1,0);
	device->BeginScene();

		mySkyBox.DrawSkybox(device,100, SkyColor);

		myXFile.Render(&tiger,device);
		myXFile2.Render(&tiger2,device);

		myBillboards2.DrawBillboard(device, myCamera.GetViewMatrix(), myCamera.GetRotationVector(), &Log);
		myBillboards4.DrawBillboard(device, myCamera.GetViewMatrix(), myCamera.GetRotationVector(), &Cloud);
		myBillboards5.DrawBillboard(device, myCamera.GetViewMatrix(), myCamera.GetRotationVector(), &Cloud1);
		myBillboards6.DrawBillboard(device, myCamera.GetViewMatrix(), myCamera.GetRotationVector(), &Cloud2);
		myBillboards7.DrawBillboard(device, myCamera.GetViewMatrix(), myCamera.GetRotationVector(), &Cloud3);


		if(FireVisible == true)
		{
			myBillboards.DrawBillboard(device, myCamera.GetViewMatrix(), myCamera.GetRotationVector(), &Fire);
		}

		if(LightningVisible == true)
		{
			myBillboards3.DrawBillboard(device, myCamera.GetViewMatrix(), myCamera.GetRotationVector(), &Lightning);
		}

		myParticleSystem.RenderPointSprites(&Smoke, device);
		myParticleSystem2.RenderPointSprites(&Rain, device);
		myParticleSystem3.RenderPointSprites(&Explotion, device);

	device->EndScene();
	device->Present(NULL,NULL,NULL,NULL);

	return true;
}


void CGame::CheckKeyboardInput(long elapsed_ms)
{	
	bool	changeview	= false;

	BYTE keyStateArray[256]; 
	
	HRESULT r = DIERR_INPUTLOST;

	while(r == DIERR_INPUTLOST || r == DIERR_NOTACQUIRED)
	{
		r = keyboard->GetDeviceState(256,keyStateArray);

		if(r == DIERR_INPUTLOST || r == DIERR_NOTACQUIRED) 
		{
			keyboard->Acquire();
		}

	}

	if(r == DI_OK)
	{
	    float movement = (float)elapsed_ms/(keyboardspeed);

		// insert key presses here
		if(keyStateArray[DIK_A] & 0x80)
		{
			myCamera.StrafeLeft(movement);
			changeview = true;
		}
		if(keyStateArray[DIK_D] & 0x80)
		{
			myCamera.StrafeRight(movement);
			changeview = true;
		}
		if(keyStateArray[DIK_W] & 0x80)
		{
			myCamera.MoveForward(movement);
			changeview = true;
		}
		if(keyStateArray[DIK_S] & 0x80)
		{
			myCamera.MoveBackward(movement);
			changeview = true;
		}

		if(keyStateArray[DIK_R] & 0x80)
		{
			myCamera.MoveUp(movement);
			changeview = true;
		}

		if(keyStateArray[DIK_V] & 0x80)
		{
			myCamera.MoveDown(movement);
			changeview = true;
		}

		D3DXVECTOR3 pos = myCamera.GetPositionVector();

		if(pos.y <= .25)
		{
			myCamera.MoveCameraToLocation(pos.x,.25,pos.z);
		}

		if(changeview == true)
		{
			myCamera.UpdateView(device);
		}
	}
	
}


void CGame::CheckMouseInput()
{
	bool	changeview	= false;

	DIMOUSESTATE2		mouseState2;   
	
	HRESULT r = DIERR_INPUTLOST; 

	while(r == DIERR_INPUTLOST || r == DIERR_NOTACQUIRED)
	{
		r = mouse->GetDeviceState(sizeof(mouseState2),&mouseState2);

		if(r == DIERR_INPUTLOST || r == DIERR_NOTACQUIRED)
		{
			r = mouse->Acquire();
		}
	}

	if(r == DI_OK)
	{
		// insert button presses here
		if(mouseState2.rgbButtons[0] & 0x80)
		{

		}
		
		// mouse movements relative to last 
		if(mouseState2.lX)
		{
			float moveX = (float) mouseState2.lX / sensitivity; 

			myCamera.RotateYAxis(moveX);

			changeview = true;
		}

		if(mouseState2.lY)
		{
			float moveY = (float) mouseState2.lY / sensitivity; 

			D3DXVECTOR3 rot = myCamera.GetRotationVector();

			if((rot.x) > .75 && moveY < 0)
			{
				moveY = 0;
			}

			if((rot.x) < -.75 && moveY > 0)
			{
				moveY = 0;
			}

			myCamera.RotateXAxis(moveY);

			changeview = true;
		}
		
		if(changeview == true)
		{
			myCamera.UpdateView(device);
		}

	}
}
