// InputEvents.cpp: implementation of the CInputEvents class.
//
//////////////////////////////////////////////////////////////////////

#include "InputEvents.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInputEvents::CInputEvents()
{
	sensitivity = 350;
	keyboardspeed = 125;
}

CInputEvents::~CInputEvents()
{
	keyboard.Unacquire();
	mouse.Unacquire();

	SAFE_RELEASE(dInput);
	SAFE_RELEASE(keyboard);
	SAFE_RELEASE(mouse);

}


bool CInputEvents::InitializeInput(HINSTANCE hInstance, HWND hwnd, CCamera *myCamera)
{

	tempCamera = &*myCamera;


	if(FAILED(DirectInput8Create(hInstance,DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&dInput,0)))
	{
		return false;
	}

	if(FAILED(dInput->CreateDevice(GUID_SysKeyboard, &keyboard, NULL)))
	{
		return false;
	}

	if(FAILED(dInput->CreateDevice(GUID_SysMouse, &mouse, NULL)))
	{
		return false;
	}

	keyboard->SetDataFormat(&c_dfDIKeyboard);

	keyboard->SetCooperativeLevel(hwnd,DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);

	mouse->SetDataFormat(&c_dfDIMouse2);

	mouse->SetCooperativeLevel(hwnd, DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);

	keyboard->Acquire();
	mouse->Acquire();


	return true;

}


void CInputEvents::CheckKeyboardInput(LPDIRECT3DDEVICE8 device, long elapsed_ms)
{	
	bool	changeview	= false;

	BYTE keyStateArray[256]; 
	
	HRESULT r = DIERR_INPUTLOST;

	while(r == DIERR_INPUTLOST || r == DIERR_NOTACQUIRED)
	{
		r = keyboard->GetDeviceState(256,keyStateArray);

		if(r == DIERR_INPUTLOST || r == DIERR_NOTACQUIRED) 
		{
			keyboard->Acquire();
		}

	}

	if(r == DI_OK)
	{
		// insert key presses here
		if(keyStateArray[DIK_A] & 0x80)
		{
			tempCamera->StrafeLeft((float)elapsed_ms/(keyboardspeed + 25));
			changeview = true;
		}
		if(keyStateArray[DIK_D] & 0x80)
		{
			tempCamera->StrafeRight((float)elapsed_ms/(keyboardspeed + 25));
			changeview = true;
		}
		if(keyStateArray[DIK_W] & 0x80)
		{
			tempCamera->MoveForward((float)elapsed_ms/keyboardspeed);
			changeview = true;
		}
		if(keyStateArray[DIK_S] & 0x80)
		{
			tempCamera->MoveBackward((float)elapsed_ms/keyboardspeed);
			changeview = true;
		}

		if(keyStateArray[DIK_R] & 0x80)
		{
			tempCamera->MoveUp((float)elapsed_ms/keyboardspeed);
			changeview = true;
		}

		if(keyStateArray[DIK_V] & 0x80)
		{
			tempCamera->MoveDown((float)elapsed_ms/keyboardspeed);
			changeview = true;
		}
		
		if(changeview == true)
		{
			tempCamera->UpdateView(device);
		}
	}
	
}


void CInputEvents::CheckMouseInput(LPDIRECT3DDEVICE8 device)
{
	bool	changeview	= false;

	DIMOUSESTATE2		mouseState2;   
	
	HRESULT r = DIERR_INPUTLOST; 

	while(r == DIERR_INPUTLOST || r == DIERR_NOTACQUIRED)
	{
		r = mouse->GetDeviceState(sizeof(mouseState2),&mouseState2);

		if(r == DIERR_INPUTLOST || r == DIERR_NOTACQUIRED)
		{
			r = mouse->Acquire();
		}
	}

	if(r == DI_OK)
	{
		// insert button presses here
		if(mouseState2.rgbButtons[0] & 0x80)
		{

		}
		
		// mouse movements relative to last 
		if(mouseState2.lX)
		{
			float moveX = (float) mouseState2.lX / sensitivity; 

			tempCamera->RotateYAxis(moveX);

			changeview = true;
		}

		if(mouseState2.lY)
		{
			float moveY = (float) mouseState2.lY / sensitivity; 

			tempCamera->RotateXAxis(moveY);

			changeview = true;
		}
		
		if(changeview == true)
		{
			tempCamera->UpdateView(device);
		}

	}
}

