// ParticleSystem.cpp: implementation of the CParticleSystem class.
//
//////////////////////////////////////////////////////////////////////

#include "ParticleSystem.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CParticleSystem::CParticleSystem()
{
	NumParticles = 0;
	time_last	 = 0;
	
	srand( (unsigned)time( NULL ) );
}

CParticleSystem::~CParticleSystem()
{
    while(ParticleList)
	{
        Particle *p = ParticleList;
        ParticleList = p->Next;
        SAFE_DELETE(p);
    }
    while(ParticlesFree)
	{
        Particle *p = ParticlesFree;
        ParticlesFree = p->Next;
        SAFE_DELETE(p);
    }

	vertexBuffer = NULL;
}


bool CParticleSystem::Initialize(LPDIRECT3DDEVICE8 device, ParticleEmitter *Emitter)
{
	time_last		= timeGetTime();

	//1000 = max number of particles
	device->CreateVertexBuffer(Emitter->MaxParticles * sizeof(structVertexBuffer), D3DUSAGE_DYNAMIC | D3DUSAGE_WRITEONLY |
								D3DUSAGE_POINTS, D3DFVF_XYZ | D3DFVF_DIFFUSE, D3DPOOL_DEFAULT, &vertexBuffer);

	D3DXCreateTextureFromFileEx(device, Emitter->TextureFilename, D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT,
								0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,D3DX_DEFAULT, D3DX_DEFAULT,D3DCOLOR_XRGB(0,0,0), NULL, 
								NULL, &particleTexture);


	return true;
}


bool CParticleSystem::EmitParticles(ParticleEmitter *Emitter)
{
	if(Emitter->Active == false) return true;
	if(Emitter->NumParticlesToEmit == 0) return true;
	if(NumParticles >= Emitter->MaxParticles) return true;

	
	long			time_current = timeGetTime();
	long			elapsed_ms   = time_current - time_last;

	int				startTime;
	float			RandVelocity;
	int				ParticlesEmitted = 0;

    Particle		*tempParticle    = NULL;

	D3DXVECTOR3		MaxDirection = Emitter->MaxRandParticleDirection * 1000;
	D3DXVECTOR3     MinDirection = Emitter->MinRandParticleDirection * 1000;

	D3DXVECTOR3		tempRadius = Emitter->Radius * 100;

	float			tempMaxRandVelocity = Emitter->MaxRandVelocity * 1000;
	float			tempMinRandVelocity = Emitter->MinRandVelocity * 1000;

	if(elapsed_ms < Emitter->Delay) 
	{
		return true;
	}
	else
	{
		time_last = time_current;
	}

	while(NumParticles < Emitter->MaxParticles && ParticlesEmitted < Emitter->NumParticlesToEmit)
	{

		if(ParticlesFree)
		{
			tempParticle = ParticlesFree;
			ParticlesFree = tempParticle->Next;
		}
		else
		{
			if(NULL == (tempParticle = new Particle()))
				return false;
		}

		tempParticle->Next = ParticleList;
		ParticleList = tempParticle;

		NumParticles++;
		ParticlesEmitted++;

		tempParticle->CurrentPosition = Emitter->Position;

		if(tempRadius.x != 0)
			tempParticle->CurrentPosition.x +=  (float)(rand() % (int)tempRadius.x) / 100;

		if(tempRadius.y != 0)
			tempParticle->CurrentPosition.y +=  (float)(rand() % (int)tempRadius.y) / 100;

		if(tempRadius.z != 0)
			tempParticle->CurrentPosition.z +=  (float)(rand() % (int)tempRadius.z) / 100;

		// check for 0
		if(MaxDirection.x - MinDirection.x == 0 )
			MaxDirection.x += 1;

		if(MaxDirection.y - MinDirection.y == 0 )
			MaxDirection.y += 1;

		if(MaxDirection.z - MinDirection.z == 0 )
			MaxDirection.z += 1;

		//random number between -1 and 1... or whatever is specified between 
		float directionx = (float)((rand() % (int)(MaxDirection.x - MinDirection.x))
									+ (int)MinDirection.x) / 1000;

		float directiony = (float)((rand() % (int)(MaxDirection.y - MinDirection.y))
									+ (int)MinDirection.y) / 1000;

		float directionz = (float)((rand() % (int)(MaxDirection.z - MinDirection.z))
									+ (int)MinDirection.z) / 1000;


		if(Emitter->MaxRandVelocity - Emitter->MinRandVelocity == 0)
		{
			RandVelocity = Emitter->MaxRandVelocity;
		}
		else
		{
			RandVelocity = (float)(rand() % ((int)tempMaxRandVelocity - (int)tempMinRandVelocity) 
							+ tempMinRandVelocity) / 1000;
		}

		//multiply velocity by direction
		tempParticle->CurrentVelocity.x = (float)(directionx * RandVelocity);
		tempParticle->CurrentVelocity.y = (float)(directiony * RandVelocity);
		tempParticle->CurrentVelocity.z = (float)(directionz * RandVelocity);

		tempParticle->CurrentColor = Emitter->StartColor;

		tempParticle->Gravity = Emitter->Gravity;

		startTime= timeGetTime();

		tempParticle->StartTime = startTime;
		tempParticle->ElapsedTime  = 0;
		tempParticle->MaxLifeTime = Emitter->MaxLifeTime;

	}

	return true;

}

bool CParticleSystem::UpdateParticles(ParticleEmitter *Emitter, float Time)
{

	float ParticleMovement = (20000) / Time;

	//update the particles
	Particle **ppParticleList = &ParticleList;
    Particle  *tempParticle = NULL;
    while(*ppParticleList)
	{

        tempParticle = *ppParticleList;

		int	currentTime = timeGetTime();

		int elapsedTime = currentTime - tempParticle->StartTime;

        //remove expired particles as we go
        if( elapsedTime > tempParticle->MaxLifeTime)
		{
            *ppParticleList    = tempParticle->Next;
            tempParticle->Next = ParticlesFree;
            ParticlesFree  = tempParticle;
            NumParticles--;
        }
        else
		{	
			tempParticle->CurrentVelocity.x = tempParticle->CurrentVelocity.x * (1 - Emitter->AirResitance / ParticleMovement);
			tempParticle->CurrentVelocity.y = tempParticle->CurrentVelocity.y * (1 - Emitter->AirResitance / ParticleMovement);
			tempParticle->CurrentVelocity.z = tempParticle->CurrentVelocity.z * (1 - Emitter->AirResitance / ParticleMovement);

			tempParticle->CurrentVelocity.y -= tempParticle->Gravity / ParticleMovement;

			tempParticle->Gravity +=  2 * tempParticle->Gravity / ParticleMovement;

			// Movement
			tempParticle->CurrentPosition.x += tempParticle->CurrentVelocity.x / ParticleMovement;
			tempParticle->CurrentPosition.y += tempParticle->CurrentVelocity.y / ParticleMovement;
			tempParticle->CurrentPosition.z += tempParticle->CurrentVelocity.z / ParticleMovement;
			
			//color change
			if(tempParticle->CurrentColor.r - Emitter->EndColor.r > 0)
			{
				tempParticle->CurrentColor.r -= Emitter->ColorFade.r * 10/ ParticleMovement;
				if(tempParticle->CurrentColor.r < Emitter->EndColor.r)
					tempParticle->CurrentColor.r = Emitter->EndColor.r;
			}
			else 
			{
				tempParticle->CurrentColor.r += Emitter->ColorFade.r * 10 / ParticleMovement;
				if(tempParticle->CurrentColor.r >= Emitter->EndColor.r)
					tempParticle->CurrentColor.r = Emitter->EndColor.r;
			}

			if(tempParticle->CurrentColor.g - Emitter->EndColor.g > 0)
			{
				tempParticle->CurrentColor.g -= Emitter->ColorFade.g * 10 / ParticleMovement;
				if(tempParticle->CurrentColor.g < Emitter->EndColor.g)
					tempParticle->CurrentColor.g = Emitter->EndColor.g;
			}
			else 
			{
				tempParticle->CurrentColor.g += Emitter->ColorFade.g * 10 / ParticleMovement;
				if(tempParticle->CurrentColor.g >= Emitter->EndColor.g)
					tempParticle->CurrentColor.g = Emitter->EndColor.g;
			}

			if(tempParticle->CurrentColor.b - Emitter->EndColor.b > 0)
			{
				tempParticle->CurrentColor.b -= Emitter->ColorFade.b * 10 / ParticleMovement;
				if(tempParticle->CurrentColor.b < Emitter->EndColor.b)
					tempParticle->CurrentColor.b = Emitter->EndColor.b;
			}
			else 
			{
				tempParticle->CurrentColor.b += Emitter->ColorFade.b * 10 / ParticleMovement;
				if(tempParticle->CurrentColor.b >= Emitter->EndColor.b)
					tempParticle->CurrentColor.b = Emitter->EndColor.b;
			}

			if(tempParticle->CurrentColor.a - Emitter->EndColor.a > 0)
			{
				tempParticle->CurrentColor.a -= Emitter->ColorFade.a * 10 / ParticleMovement;
				if(tempParticle->CurrentColor.a < Emitter->EndColor.a)
					tempParticle->CurrentColor.a = Emitter->EndColor.a;
			}
			else 
			{
				tempParticle->CurrentColor.a += Emitter->ColorFade.a * 10 / ParticleMovement;
				if(tempParticle->CurrentColor.a >= Emitter->EndColor.a)
					tempParticle->CurrentColor.a = Emitter->EndColor.a;
			}


			tempParticle->ElapsedTime = currentTime - tempParticle->StartTime;

            ppParticleList = &tempParticle->Next;
		}
    }

	//emit new particles
	EmitParticles(Emitter);	

	return true;
}

bool CParticleSystem::RenderPointSprites(ParticleEmitter *Emitter, LPDIRECT3DDEVICE8 device)
{
	int numParticles = 0;

	Particle **ppParticleList = &ParticleList;
    Particle  *tempParticle = NULL;

	tempParticle = *ppParticleList;

    //set render states
	D3DXMATRIX mat = GetIdentityMatrix();
	device->SetTransform(D3DTS_WORLD, &mat);  //location 0,0,0 

	device->SetRenderState(D3DRS_LIGHTING, FALSE);
    device->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);
	device->SetRenderState( D3DRS_POINTSPRITEENABLE, TRUE );
    device->SetRenderState( D3DRS_POINTSCALEENABLE,  TRUE );

	device->SetRenderState( D3DRS_POINTSIZE, *(DWORD *)&Emitter->ParticleSize);
    device->SetRenderState( D3DRS_POINTSIZE_MIN, 0 ); 
    device->SetRenderState( D3DRS_POINTSCALE_A, *(DWORD *)&Emitter->ScaleX);
    device->SetRenderState( D3DRS_POINTSCALE_B, *(DWORD *)&Emitter->ScaleY);
    device->SetRenderState( D3DRS_POINTSCALE_C, *(DWORD *)&Emitter->ScaleZ); 

//	device->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

	device->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE); 

	//disable detail mapping
	device->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);

	device->SetTexture(0, particleTexture);

	device->SetStreamSource(0, vertexBuffer, sizeof(structVertexBuffer));

	//get a pointer to the buffer
    structVertexBuffer	*vertexNode = NULL;

	vertexBuffer->Lock(0,numParticles, (BYTE**)&vertexNode, D3DLOCK_DISCARD);

	//fill vertNodes
    while(tempParticle)
	{
		vertexNode->posx  = tempParticle->CurrentPosition.x;
		vertexNode->posy  = tempParticle->CurrentPosition.y;
		vertexNode->posz  = tempParticle->CurrentPosition.z;
		vertexNode->color = tempParticle->CurrentColor;
		vertexNode++;

		//get next particle in list
        tempParticle = tempParticle->Next;

		numParticles++;
    }

	//unlock the vertex buffer
	vertexBuffer->Unlock();

	if(numParticles != 0)
	{
		device->DrawPrimitive(D3DPT_POINTLIST, 0, numParticles); 
	}

    //reset render states
	device->SetRenderState(D3DRS_LIGHTING, TRUE);
    device->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
	device->SetRenderState( D3DRS_POINTSPRITEENABLE, FALSE);
    device->SetRenderState( D3DRS_POINTSCALEENABLE, FALSE);	
	device->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);


	//enable detail mapping
	device->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_MODULATE2X);

	//enable textures
	device->SetTextureStageState(0,D3DTSS_COLOROP, D3DTOP_MODULATE);


	//POSSIBLE MEMORY LEAKS
	//vertexbuffer... im not sure if D3DLOCK_DISCARD is enough
	//vertexNode.. if i have to delete that as well

    return true;
}