// ParticleSystem.h: interface for the CParticleSystem class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PARTICLESYSTEM_H
#define AFX_PARTICLESYSTEM_H

#include <d3d8.h>
#include <d3dx8.h>

#include <time.h>
#include "structs.h"
#include "defines.h"
#include "matrixcalls.h"


class CParticleSystem  
{

struct Particle
{
	float			StartTime;
	float			ElapsedTime;
	float			MaxLifeTime;

	D3DXVECTOR3		CurrentPosition;
	D3DXVECTOR3		CurrentVelocity;
	D3DXCOLOR		CurrentColor;

	float			Gravity;

	Particle *		Next;

	Particle()
	{
		Next = NULL;
	}

};

struct structVertexBuffer
{
	float		posx;
	float		posy;
	float		posz;
	DWORD		color;
};


public:
	CParticleSystem();
	virtual ~CParticleSystem();
	bool EmitParticles(ParticleEmitter *Emitter);
	bool UpdateParticles(ParticleEmitter *Emitter, float Time);
	bool RenderPointSprites(ParticleEmitter *Emitter, LPDIRECT3DDEVICE8 device);
	bool Initialize(LPDIRECT3DDEVICE8 device, ParticleEmitter *Emitter);

private:
	int				NumParticles;

	Particle *		ParticleList;
	Particle *		ParticlesFree;

	LPDIRECT3DVERTEXBUFFER8	vertexBuffer;

	LPDIRECT3DTEXTURE8	particleTexture;

	long	time_last;

};

#endif 
