// RenderStates.cpp: implementation of the CRenderStates class.
//
//////////////////////////////////////////////////////////////////////

#include "RenderStates.h"

#pragma warning( disable : 4244 )

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRenderStates::CRenderStates()
{
	LightNumber = -1;

	ScalingMatrix= GetIdentityMatrix();

	ScalingMatrix._11 = 8;
	ScalingMatrix._22 = 8;

}

CRenderStates::~CRenderStates()
{

}

bool CRenderStates::InitializeRenderStates(LPDIRECT3DDEVICE8 device)
{
	device->SetRenderState(D3DRS_LIGHTING, TRUE);
	device->SetRenderState(D3DRS_COLORVERTEX, TRUE);
	//dont cull faces
	device->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	//use material for calculating diffuse lighting
	device->SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE, D3DMCS_MATERIAL);

	//alpha blending
	device->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	device->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	device->SetRenderState(D3DRS_DESTBLEND,	D3DBLEND_INVSRCALPHA);


	//STAGE 0

	//bilinear filtering
	device->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	device->SetTextureStageState(0,	D3DTSS_MINFILTER, D3DTEXF_LINEAR);

	//wrap textures
	device->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
	device->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);

	//mipmapping
	device->SetTextureStageState(0,D3DTSS_MIPFILTER, D3DTEXF_POINT);
	device->SetTextureStageState(0,D3DTSS_MIPMAPLODBIAS, -1.5f);

	//combine texture with diffuse lighting
	device->SetTextureStageState(0,D3DTSS_COLORARG1, D3DTA_TEXTURE);
	device->SetTextureStageState(0,D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	device->SetTextureStageState(0,D3DTSS_COLOROP, D3DTOP_MODULATE);

	device->SetTextureStageState(0,D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	device->SetTextureStageState(0,D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	device->SetTextureStageState(0,D3DTSS_ALPHAOP, D3DTOP_MODULATE);



	//STAGE 1

	//bilinear filtering
	device->SetTextureStageState(1, D3DTSS_MAGFILTER, D3DTEXF_POINT);
	device->SetTextureStageState(1,	D3DTSS_MINFILTER, D3DTEXF_POINT);

	//wrap textures
	device->SetTextureStageState(1, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
	device->SetTextureStageState(1, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);

	//mipmapping
	device->SetTextureStageState(1,D3DTSS_MIPFILTER, D3DTEXF_POINT);
	device->SetTextureStageState(1,D3DTSS_MIPMAPLODBIAS, -.001f);

	//detail mapping
	device->SetTextureStageState(1, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2);
	device->SetTransform(D3DTS_TEXTURE1, &ScalingMatrix); 

	device->SetTextureStageState(1, D3DTSS_TEXCOORDINDEX, 0);

	device->SetTextureStageState(1,D3DTSS_COLORARG1, D3DTA_TEXTURE);
	device->SetTextureStageState(1,D3DTSS_COLORARG2, D3DTA_CURRENT);
	device->SetTextureStageState(1,D3DTSS_COLOROP, D3DTOP_MODULATE2X); // Set to Add not modulate my videocard does not support it. 


	return true;
}


void CRenderStates::CreateLight(LightObject *myLight, LPDIRECT3DDEVICE8 device)
{

	LightNumber++;

	myLight->LightNumber		= LightNumber;
    myLight->Light.Type			= myLight->Type;
    myLight->Light.Direction    = myLight->Direction;
    myLight->Light.Position		= myLight->Position;
    myLight->Light.Diffuse      = myLight->Diffuse;
    myLight->Light.Ambient      = myLight->Ambient;
    myLight->Light.Range        = myLight->Range;
    myLight->Light.Attenuation0 = myLight->Attenuation0;
    myLight->Light.Attenuation1 = myLight->Attenuation1;
    myLight->Light.Attenuation2 = myLight->Attenuation2;
    myLight->Light.Falloff		= myLight->Falloff;
    myLight->Light.Theta        = myLight->Theta;  //inner cone angle
    myLight->Light.Phi          = myLight->Phi;  //outer cone angle


	device->SetLight(LightNumber,&myLight->Light);

	device->LightEnable(LightNumber,TRUE);
}

void CRenderStates::LightOn(int LightNumber, LPDIRECT3DDEVICE8 device)
{
	device->LightEnable(LightNumber,TRUE);
}

void CRenderStates::LightOff(int LightNumber, LPDIRECT3DDEVICE8 device)
{
	device->LightEnable(LightNumber,FALSE);
}