// SkyBox.cpp: implementation of the CSkyBox class.
//
//////////////////////////////////////////////////////////////////////

#include "SkyBox.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSkyBox::CSkyBox()
{
	identity  = GetIdentityMatrix();
}

CSkyBox::~CSkyBox()
{
	//Release SkyBox textures
	for(int i = 0; i < Textures.size(); i++)
		SAFE_RELEASE(Textures.at(i));
}

bool CSkyBox::InitializeSkybox(LPDIRECT3DDEVICE8 device,char* filenameFront,char* filenameBack,
								char* filenameLeft,char* filenameRight,char* filenameUp,char* filenameDown)
{
	std::vector<char *> TextureFilenames;

	TextureFilenames.push_back(filenameFront);
	TextureFilenames.push_back(filenameBack);
	TextureFilenames.push_back(filenameLeft);
	TextureFilenames.push_back(filenameRight);
	TextureFilenames.push_back(filenameUp);
	TextureFilenames.push_back(filenameDown);

	LPDIRECT3DTEXTURE8 temp;

	for(int i = 0; i < TextureFilenames.size(); i ++)
	{
		HRESULT hr = D3DXCreateTextureFromFile(device, TextureFilenames.at(i), &temp);
		if(hr != D3D_OK)
		{
			LogError("Skybox texture at %s did not load", TextureFilenames.at(i));

			temp = NULL;
		}

		Textures.push_back(temp);
	}

	return true;
}

bool CSkyBox::DrawSkybox(LPDIRECT3DDEVICE8 device, float Size, DWORD Color)
{
	//set world matrix to identity
	device->SetTransform(D3DTS_WORLD, &identity);

	//turn off lighting
	device->SetRenderState(D3DRS_LIGHTING, FALSE);

	//turn off depth buffer writes
	device->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);

	//set the vertex streams
	device->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1);

	//clamp textures
	device->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
	device->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_CLAMP);

	//disable detail mapping
	device->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);

	//alphablending detail mapping
	device->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);


	LitVertex Mesh[4];

	if(Textures.at(0) != NULL)
	{
		//front
		Mesh[0].x = Size; Mesh[0].y = Size;  Mesh[0].z = -Size;
		Mesh[0].u = 0.0f;     Mesh[0].v = 0.0f;
		Mesh[0].color = Color;

		Mesh[1].x = -Size;  Mesh[1].y = Size;	Mesh[1].z = -Size;
		Mesh[1].u = 1.0f;     Mesh[1].v = 0.0f;
		Mesh[1].color = Color;

		Mesh[2].x = Size;  Mesh[2].y = -Size; Mesh[2].z = -Size;
		Mesh[2].u = 0.0f;     Mesh[2].v = 1.0f;
		Mesh[2].color = Color;

		Mesh[3].x = -Size; Mesh[3].y = -Size; Mesh[3].z = -Size;
		Mesh[3].u = 1.0f;     Mesh[3].v = 1.0f;
		Mesh[3].color = Color;
		
		device->SetTexture(0, Textures.at(0));
		device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));
	}

	if(Textures.at(1) != NULL)
	{

		//back
		Mesh[0].x = -Size; Mesh[0].y = Size;  Mesh[0].z = Size;
		Mesh[0].u = 0.0f;  Mesh[0].v = 0.0f;
		Mesh[0].color = Color;

		Mesh[1].x = Size;  Mesh[1].y = Size;	Mesh[1].z = Size;
		Mesh[1].u = 1.0f;  Mesh[1].v = 0.0f;
		Mesh[1].color = Color;

		Mesh[2].x = -Size;  Mesh[2].y = -Size; Mesh[2].z = Size;
		Mesh[2].u = 0.0f;   Mesh[2].v = 1.0f;
		Mesh[2].color = Color;

		Mesh[3].x = Size; Mesh[3].y = -Size; Mesh[3].z = Size;
		Mesh[3].u = 1.0f; Mesh[3].v = 1.0f;
		Mesh[3].color = Color;
		
		device->SetTexture(0, Textures.at(1));
		device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));

	}

	if(Textures.at(2) != NULL)
	{
		//left
		Mesh[0].x = -Size;  Mesh[0].y = Size;  Mesh[0].z = -Size;
		Mesh[0].u = 0.0f;   Mesh[0].v = 0.0f;
		Mesh[0].color = Color;

		Mesh[1].x = -Size;  Mesh[1].y = Size;  Mesh[1].z = Size;
		Mesh[1].u = 1.0f;   Mesh[1].v = 0.0f;
		Mesh[1].color = Color;

		Mesh[2].x = -Size;  Mesh[2].y = -Size; Mesh[2].z = -Size;
		Mesh[2].u = 0.0f;   Mesh[2].v = 1.0f;
		Mesh[2].color = Color;

		Mesh[3].x = -Size;  Mesh[3].y = -Size; Mesh[3].z = Size;
		Mesh[3].u = 1.0f;   Mesh[3].v = 1.0f;
		Mesh[3].color = Color;

		device->SetTexture(0, Textures.at(2));
		device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));
	}

	if(Textures.at(3) != NULL)
	{
		//right
		Mesh[0].x = Size; Mesh[0].y = Size;   Mesh[0].z = Size;
		Mesh[0].u = 0.0f; Mesh[0].v = 0.0f;
		Mesh[0].color = Color;

		Mesh[1].x = Size;  Mesh[1].y = Size;  Mesh[1].z = -Size;
		Mesh[1].u = 1.0f;  Mesh[1].v = 0.0f;
		Mesh[1].color = Color;

		Mesh[2].x = Size;  Mesh[2].y = -Size; Mesh[2].z = Size;
		Mesh[2].u = 0.0f;  Mesh[2].v = 1.0f;
		Mesh[2].color = Color;

		Mesh[3].x = Size; Mesh[3].y = -Size;  Mesh[3].z = -Size;
		Mesh[3].u = 1.0f; Mesh[3].v = 1.0f;
		Mesh[3].color = Color;

		device->SetTexture(0, Textures.at(3));
		device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));
	}

	if(Textures.at(4) != NULL)
	{
		//up
		Mesh[0].x = Size; Mesh[0].y = Size;  Mesh[0].z = -Size;
		Mesh[0].u = 0.0f;     Mesh[0].v = 0.0f;
		Mesh[0].color = Color;

		Mesh[1].x = -Size;  Mesh[1].y = Size;	Mesh[1].z = -Size;
		Mesh[1].u = 0.0f;     Mesh[1].v = 1.0f;
		Mesh[1].color = Color;

		Mesh[2].x = Size; Mesh[2].y = Size; Mesh[2].z = Size;
		Mesh[2].u = 1.0f;     Mesh[2].v = 0.0f;
		Mesh[2].color = Color;

		Mesh[3].x = -Size;  Mesh[3].y = Size; Mesh[3].z = Size;
		Mesh[3].u = 1.0f;     Mesh[3].v = 1.0f;
		Mesh[3].color = Color;

		device->SetTexture(0, Textures.at(4));
		device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));
	}

	if(Textures.at(5) != NULL)
	{
		//down
		Mesh[0].x = -Size;  Mesh[0].y  = -Size; Mesh[0].z = -Size;
		Mesh[0].u = 0.0f;     Mesh[0].v = 0.0f;
		Mesh[0].color = Color;

		Mesh[1].x = -Size;  Mesh[1].y = -Size; Mesh[1].z = Size;
		Mesh[1].u = 1.0f;      Mesh[1].v = 0.0f;
		Mesh[1].color = Color;

		Mesh[2].x = Size;  Mesh[2].y  = -Size; Mesh[2].z = -Size;
		Mesh[2].u = 0.0f;     Mesh[2].v = 1.0f;
		Mesh[2].color = Color;

		Mesh[3].x = Size;  Mesh[3].y = -Size; Mesh[3].z = Size;
		Mesh[3].u = 1.0f;      Mesh[3].v = 1.0f;
		Mesh[3].color = Color;

		device->SetTexture(0, Textures.at(5));
		device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));
	}

	//restore settings
	device->SetRenderState(D3DRS_LIGHTING, TRUE);
	device->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
	device->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
	device->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);
	device->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_MODULATE2X);

	device->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

	return true;
}
