// XFile.cpp: implementation of the CXFile class.
//
//////////////////////////////////////////////////////////////////////

#include "XFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXFile::CXFile()
{

}

CXFile::~CXFile()
{
	for(int i = 0; i < textures.size() - 1; i++)
		SAFE_RELEASE(textures.at(i));

	SAFE_RELEASE(mesh);

}

bool CXFile::LoadXFile(XObject *object, LPDIRECT3DDEVICE8 device)
{  

	D3DXMATERIAL*		tempMaterials;
	DWORD               NumMaterials; 
	LPD3DXBUFFER        bufMaterials;   
    LPD3DXBUFFER        bufAdjacency;


	LPD3DXMESH temp		  = NULL;
	DWORD	desired_FVF  = D3DFVF_TEX2 | D3DFVF_DIFFUSE | D3DFVF_XYZ
						| D3DFVF_NORMAL; 

	HRESULT hr = D3DXLoadMeshFromX(object->filename, D3DXMESH_MANAGED, device, 
					               &bufAdjacency, &bufMaterials, &NumMaterials, 
							       &mesh);
    if(FAILED(hr))
	{
		LogError("Could not load xfile %s",object->filename);
        return false;
	}
    if(NumMaterials == 0)
	{
		LogError("No materials found for .x mesh %s",object->filename);
        return false;
    }

    
	if(mesh->GetFVF() != desired_FVF)
	{
		mesh->CloneMeshFVF(mesh->GetOptions(), desired_FVF, device, &temp);
		SAFE_DELETE(mesh);
		mesh = temp;

		D3DXComputeNormals(mesh,(DWORD *)bufAdjacency->GetBufferPointer());
	}


	mesh->OptimizeInplace(D3DXMESHOPT_ATTRSORT | D3DXMESHOPT_VERTEXCACHE,(DWORD *)bufAdjacency->GetBufferPointer()
						,NULL, NULL, NULL);

	m_numMaterials = NumMaterials;

	tempMaterials = (D3DXMATERIAL*) bufMaterials->GetBufferPointer();

	for(int i = 0; i < NumMaterials; i++)
	{
		LPDIRECT3DTEXTURE8   tempTexture;
		textures.push_back(tempTexture);

		materials.push_back(tempMaterials[i].MatD3D);

		strcat(object->filenameLoc, tempMaterials->pTextureFilename);

		if(FAILED(D3DXCreateTextureFromFileA(device, object->filenameLoc, &textures.at(i))))
		{
			tempTexture = NULL;
		}

	}

	// if object has no ambient material
	if(materials.at(0).Ambient.r == 0 && materials.at(0).Ambient.g == 0 
		&& materials.at(0).Ambient.b == 0)
	{
		materials.at(0).Ambient.r = .5f;				
		materials.at(0).Ambient.g = .5f;
		materials.at(0).Ambient.b = .5f;
		materials.at(0).Ambient.a = 1.0f;
	}

	// if object has no diffuse material 
	if(materials.at(0).Diffuse.r == 0 && materials.at(0).Diffuse.g == 0 
		&& materials.at(0).Diffuse.b == 0)
	{
		materials.at(0).Diffuse.r = 1.0f;				
		materials.at(0).Diffuse.g = 1.0f;
		materials.at(0).Diffuse.b = 1.0f;
		materials.at(0).Diffuse.a = 1.0f;
	}

	SAFE_RELEASE(bufMaterials);
	SAFE_RELEASE(bufAdjacency);

	return true;
}

bool CXFile::Render(XObject *object, LPDIRECT3DDEVICE8 device)
{

	device->SetTransform(D3DTS_WORLD, &(object->WorldMatrix));

	for(int i = 0; i < m_numMaterials; i++)
	{
		device->SetMaterial(&materials.at(i));

        device->SetTexture(0,textures.at(i));

	    mesh->DrawSubset(i);
    }	



	return true;
}

