/**********************************************
This very simple application will demonstrate
how to create a window and how to create and 
initialize a valid Direct3D rendering device.
***********************************************/


#pragma comment (lib,"dxguid.lib")
#pragma comment (lib,"d3dx8dt.lib")
#pragma comment (lib,"d3dx8.lib")
#pragma comment (lib,"d3d8.lib")
#pragma comment (lib,"dinput8.lib")
#pragma comment (lib,"dsound.lib")
#pragma comment (lib,"winmm.lib")
#pragma comment (lib,"d3dxof.lib")
#pragma comment (lib,"dxerr8.lib")

#include "Game.h"


using namespace std;

//////////////////////////////////////////////////////
// functions

HWND	CreateAppWindow(HINSTANCE hInstance, int w, int h, bool fullscreen = false);

LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

//////////////////////////////////////////////////////
// variables
bool		appRunning		  = true;

const bool  fullscreen        = true;
const int   screen_width      = 800;
const int   screen_height     = 600;
int			StartX	          = CW_USEDEFAULT;
int 		StartY            = CW_USEDEFAULT;
const char* window_header	  = "Direct 3D engine";

CGame myGame;


///////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow){


	//////////////////////////////////////////////////////
	//create a window for our application
	HWND hwnd = CreateAppWindow(hInstance,screen_width,screen_height,fullscreen);

	//////////////////////////////////////////////////////
	//create and initialize a direct3d rendering device
	if(!myGame.InitializeGame(hInstance,hwnd, screen_width, screen_height, fullscreen))
	{
		MessageBox(NULL, "Direct 3d not initialized! Try downloading directX 8.0",
			"Error", MB_OK);

		PostQuitMessage(0);
	}

	//////////////////////////////////////////////////////
	//enter main event loop
	MSG msg;		 
	while(true)
	{
		if(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
			if(msg.message == WM_QUIT)
				break;

			if(KEYDOWN(VK_ESCAPE))
				SendMessage(hwnd,WM_CLOSE,0,0);

			TranslateMessage(&msg);
		    DispatchMessage(&msg);
		} 
		
		if(appRunning == true)
		{
			myGame.Process();
		}

	}

	//return to Windows
	return(msg.wParam);
}

//////////////////////////////////////////
// Main Window Message Handler
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	switch(msg)
	{
		case WM_CLOSE: 
			//cleanup direct3d
			DestroyWindow(hwnd);
			PostQuitMessage(0);
			appRunning = false;
		break;
		default:
			return DefWindowProc( hwnd, msg, wparam, lparam );
	}

	return DefWindowProc( hwnd, msg, wparam, lparam );
} 


/////////////////////////////////////////////////
//Window Initialization
HWND CreateAppWindow(HINSTANCE hInstance, int w, int h, bool fullscreen)
{

	int   WindowWidth  = w;
	int   WindowHeight = h;
	int   ScreenWidth  = GetSystemMetrics(SM_CXSCREEN);
	int   ScreenHeight = GetSystemMetrics(SM_CYSCREEN);

   	DWORD Style = WS_POPUP | WS_VISIBLE;
	RECT rc;
    
	HWND  main_window_handle = NULL;

	//insert menu here
   	HMENU hmenu = NULL;

	//first fill in the window class stucture
	WNDCLASSEX winclass;
	memset(&winclass,0,sizeof(WNDCLASSEX));
	winclass.cbSize 	 = sizeof(WNDCLASSEX);
	winclass.style		 = CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc = WindowProc;
	winclass.cbClsExtra	 = 0;
	winclass.cbWndExtra	 = 0;
	winclass.hInstance	 = hInstance;
	winclass.hIcon		 = LoadIcon(NULL, IDI_APPLICATION);
	winclass.hIconSm     = LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor	 = LoadCursor(NULL, IDC_ARROW); 
	winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL;
	winclass.lpszClassName	= "My Engine";

	//register the window class
	if(!RegisterClassEx(&winclass))
		return NULL;

	//make any last minute adjustments for windowed vs. full-screen mode
    if(!fullscreen){
  
		Style |= WS_OVERLAPPEDWINDOW;
        
		SetRect(&rc, 0, 0, WindowWidth, WindowHeight);
		AdjustWindowRect(&rc, Style, TRUE);
        ScreenWidth  = rc.right-rc.left;
        ScreenHeight = rc.bottom-rc.top;
	}

	//create the render window
	main_window_handle = CreateWindow( "My Engine", 
									   window_header,
									   Style,
									   StartX,
									   StartY,
									   ScreenWidth,
									   ScreenHeight,
									   0L,
									   hmenu, 
									   hInstance,
									   0L );
	if(!main_window_handle)
		return NULL;

	//update the window
	if(!fullscreen)
		UpdateWindow(main_window_handle);

	return main_window_handle;
}


